REBOL [
	Title: "REBOL/Services - Server API Functions"
	Author: "Carl Sassenrath"
	Notice: "Confidential source code. Under development."
	Copyright: "REBOL Technology 2005-2008, All rights reserved."
]

;-- Server API Functions -----------------------------------------------------

start-service: func [
	"Starts a REBOL Service to receive requests. Returns service object."
	port-spec [url! block! port! word!] "Service request scheme or name"
	/options config [file! block! none!] "Services configuration or services list."
	/info serv-info [block!] "Service information object"
	/local port scheme handler
][
	if info [service-info: make service-info serv-info]
	config: load-config config port-spec = 'cgi ; avoid output into CGI stream
	load-userbase config/user-file
	debug system ['start-service port-spec system/product system/version system/build]
	load-services config
	scheme: either word? port-spec [
		port: port-spec ; (not a port)
	][
		port: make port! port-spec
		port/scheme
	]
	if handler: select server-handlers scheme [
		handler/opener port config
	]
]

stop-service: func [
	"Close a REBOL Service that was opened earlier."
	port [port! none!]
][
	if not port? port [exit]
	debug system ['stop-service port/port-id]
	attempt [close port]
]

publish-service: func [
	"Publish a REBOL Service in the global service directory."
	service [object!] "From MakeService or OpenService"
	/info "When MakeService not used, info must be provided"
	data [block!] "Needed to publish: [name-str email-contact]"
][
	debug system ['publish]
	not-supported-yet
]
