REBOL [
	Title: "REBOL/Services - File Service Dialect"
	Author: "Carl Sassenrath"
	Notice: "Confidential source code. Under development."
	Copyright: "REBOL Technology 2005-2008, All rights reserved."
]

add-default-service [
	name: 'file
	title: "Simple File Service"
	description: {
		Implements file transfer, directory listings,
		deletion (files and dirs), text search, and permissions.
	}
	base: clean-path %public/
	if not exists? base [make-dir/deep base]
	filename: contents: offset: range: none
	commands: [
		'get "Download a file" #need file-get
			set filename file!
			[set offset integer! set range integer! | (offset: range: none)]
			(result: get-file filename offset range)
		|
		'list "List available files" #need file-list
			set args opt file!
			(result: list-files args)
		|
		'delete "Delete a file" #need file-delete
			set args file!
			(result: delete-file args)
		| 
		; to put multiple files, use multiple put commands in the same request.
		; i'm not allowing a list of files to put because that complicates
		; error reporting.
		'put "Upload a file" #need file-put
			set filename file!
			set contents binary!
			(result: put-file filename contents)
		| 
		'append "Append data to a file" #need file-put
			set filename file!
			set contents binary!
			(result: put-file/append filename contents)
		; PROBLEM: set-modes evaluates the argument block!
		;|
		;'modes args: file! any [set-word! skip]
	]

	check-path: func [file [file!]] [
		file: clean-path base/:file
		if find/match file base [file]
	]
	public-part: func [file [file!]] [
		copy at file length? base
	]
	list-files: func [arg [file! none!] /local dir result] [
		arg: any [arg %/]
		if not dir: dirize check-path arg [
			throw ['not-allowed arg]
		]
		if not all [exists? dir dir? dir] [
			throw reduce ['no-dir arg]
		]
		if not result: attempt [read dir] [
			throw reduce ['access-error public-part dir]
		]
		forall result [
			result/1: third make info? join dir result/1 [name: result/1]
		]
		insert result public-part dir
		result
	]
	get-file: func [arg [file!] offset [integer! none!] range [integer! none!] /local port filename result date] [
		if not filename: check-path arg [
			throw reduce ['not-allowed arg]
		]
		if not exists? filename [
			throw reduce ['no-file arg]
		]
		if error? try [
			date: modified? filename
			either all [offset range] [
				port: open/binary/seek/read filename
				port: at port offset
				offset: index? port
				result: copy/part port range
				range: length? result
				close port
			] [
				result: read/binary filename
				offset: 1
				range: length? result
			]
		] [
			throw reduce ['access-error public-part filename]
		]
		reduce [public-part filename offset range result date]
	]
	delete-file: func [arg [file!] /local filename] [
		if not filename: check-path arg [
			throw reduce ['not-allowed arg]
		]
		if not exists? filename [
			throw reduce ['no-file arg]
		]
		either attempt [delete filename] [
			true
		] [
			throw reduce ['no-delete public-part filename]
		]
	]
	put-file: func [arg [file!] contents [any-string!] /append /local filename] [
		if not filename: check-path arg [
			throw reduce ['not-allowed arg]
		]
		if error? try [
			make-dir/deep first split-path filename
			either append [
				write/binary/append filename contents
			] [
				write/binary filename contents
			]
		] [
			throw reduce ['write-error public-part filename]
		]
		reduce [public-part filename length? contents size? filename]
	]
]
